@@ -3,7 +3,7 @@
  +--------------------------------------------------------------------+
  | CiviCRM version 4.7                                                |
  +--------------------------------------------------------------------+
- | Copyright CiviCRM LLC (c) 2004-2015                                |
+ | Copyright CiviCRM LLC (c) 2004-2017                                |
  +--------------------------------------------------------------------+
  | This file is a part of CiviCRM.                                    |
  |                                                                    |
@@ -28,7 +28,7 @@
 /**
  *
  * @package CRM
- * @copyright CiviCRM LLC (c) 2004-2015
+ * @copyright CiviCRM LLC (c) 2004-2017
  */
 
 require_once 'packages/When/When.php';
@@ -126,6 +126,11 @@ class CRM_Core_BAO_RecurringEntity extends CRM_Core_DAO_RecurringEntity {
       ),
     );
 
+  //Define global CLASS CONSTANTS for recurring entity mode types
+  const MODE_THIS_ENTITY_ONLY = 1;
+  const MODE_NEXT_ALL_ENTITY = 2;
+  const MODE_ALL_ENTITY_IN_SERIES = 3;
+
   /**
    * Getter for status.
    *
@@ -842,8 +847,8 @@ class CRM_Core_BAO_RecurringEntity extends CRM_Core_DAO_RecurringEntity {
           }
 
           $linkedDao = new $daoName();
-          $linkedDao->$linfo['entity_id_col'] = $entityId;
-          $linkedDao->$linfo['entity_table_col'] = $entityTable;
+          $linkedDao->{$linfo['entity_id_col']} = $entityId;
+          $linkedDao->{$linfo['entity_table_col']} = $entityTable;
           $linkedDao->find();
           while ($linkedDao->fetch()) {
             CRM_Core_BAO_RecurringEntity::delEntity($linkedDao->id, $linkedTable, FALSE);
@@ -1174,4 +1179,64 @@ class CRM_Core_BAO_RecurringEntity extends CRM_Core_DAO_RecurringEntity {
     return $result;
   }
 
+  /**
+   * Update mode in civicrm_recurring_entity table for event related data and price set in civicrm_price_set_entity.
+   *
+   * @param int $entityId
+   *   Event id .
+   * @param string $entityTable
+   * @param string $mode
+   * @param string $linkedEntityTable
+   *   Linked entity table name for this event .
+   * @param string $priceSet
+   *   Price set of the event .
+   *
+   * @return array
+   */
+  public static function updateModeAndPriceSet($entityId, $entityTable, $mode, $linkedEntityTable, $priceSet) {
+    $finalResult = array();
+
+    if (!empty($linkedEntityTable)) {
+      $result = CRM_Core_BAO_RecurringEntity::updateModeLinkedEntity($entityId, $linkedEntityTable, $entityTable);
+    }
+
+    $dao = new CRM_Core_DAO_RecurringEntity();
+    if (!empty($result)) {
+      $dao->entity_id = $result['entityId'];
+      $dao->entity_table = $result['entityTable'];
+    }
+    else {
+      $dao->entity_id = $entityId;
+      $dao->entity_table = $entityTable;
+    }
+
+    if ($dao->find(TRUE)) {
+      $dao->mode = $mode;
+      $dao->save();
+
+      //CRM-20787 Fix
+      //I am not sure about other fields, if mode = 3 apply for an event then other fields
+      //should be save for all other series events or not so applying for price set only for now here.
+      if (CRM_Core_BAO_RecurringEntity::MODE_ALL_ENTITY_IN_SERIES === $mode) {
+
+        //Step-1: Get all events of series
+        $seriesEventRecords = CRM_Core_BAO_RecurringEntity::getEntitiesFor($entityId, $entityTable);
+        foreach ($seriesEventRecords as $event) {
+          //Step-3: Save price set in other series events
+          if (CRM_Price_BAO_PriceSet::removeFrom($event['table'], $event['id'])) {//Remove existing priceset
+            CRM_Core_BAO_Discount::del($event['id'], $event['table']);
+            CRM_Price_BAO_PriceSet::addTo($event['table'], $event['id'], $priceSet); //Add new price set
+          }
+        }
+      }
+      //CRM-20787 - Fix end
+      $finalResult['status'] = 'Done';
+    }
+    else {
+      $finalResult['status'] = 'Error';
+    }
+
+    return $finalResult;
+  }
+
 }
